@@ -1,9 +1,9 @@
 /*
   zip_source_window.c -- return part of lower source
-  Copyright (C) 2012-2021 Dieter Baron and Thomas Klausner
+  Copyright (C) 2012-2024 Dieter Baron and Thomas Klausner
 
   This file is part of libzip, a library to manipulate ZIP archives.
-  The authors can be contacted at <libzip@nih.at>
+  The authors can be contacted at <info@libzip.org>
 
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
@@ -49,7 +49,10 @@ struct window {
     zip_uint64_t offset; /* offset in src for next read */
 
     zip_stat_t stat;
+    zip_uint64_t stat_invalid;
     zip_file_attributes_t attributes;
+    zip_dostime_t dostime;
+    bool dostime_valid;
     zip_error_t error;
     zip_int64_t supports;
     bool needs_seek;
@@ -60,12 +63,13 @@ static zip_int64_t window_read(zip_source_t *, void *, void *, zip_uint64_t, zip
 
 ZIP_EXTERN zip_source_t *
 zip_source_window_create(zip_source_t *src, zip_uint64_t start, zip_int64_t len, zip_error_t *error) {
-    return _zip_source_window_new(src, start, len, NULL, 0, NULL, 0, error);
+    return _zip_source_window_new(src, start, len, NULL, 0, NULL, NULL, NULL, 0, false, error);
 }
 
 
 zip_source_t *
-_zip_source_window_new(zip_source_t *src, zip_uint64_t start, zip_int64_t length, zip_stat_t *st, zip_file_attributes_t *attributes, zip_t *source_archive, zip_uint64_t source_index, zip_error_t *error) {
+_zip_source_window_new(zip_source_t *src, zip_uint64_t start, zip_int64_t length, zip_stat_t *st, zip_uint64_t st_invalid, zip_file_attributes_t *attributes, zip_dostime_t *dostime, zip_t *source_archive, zip_uint64_t source_index, bool take_ownership, zip_error_t *error) {
+    zip_source_t* window_source;
     struct window *ctx;
 
     if (src == NULL || length < -1 || (source_archive == NULL && source_index != 0)) {
@@ -94,16 +98,24 @@ _zip_source_window_new(zip_source_t *src, zip_uint64_t start, zip_int64_t length
         ctx->end_valid = true;
     }
     zip_stat_init(&ctx->stat);
+    ctx->stat_invalid = st_invalid;
     if (attributes != NULL) {
-        memcpy(&ctx->attributes, attributes, sizeof(ctx->attributes));
+        (void)memcpy_s(&ctx->attributes, sizeof(ctx->attributes), attributes, sizeof(ctx->attributes));
     }
     else {
         zip_file_attributes_init(&ctx->attributes);
     }
+    if (dostime != NULL) {
+        ctx->dostime = *dostime;
+        ctx->dostime_valid = true;
+    }
+    else {
+        ctx->dostime_valid = false;
+    }
     ctx->source_archive = source_archive;
     ctx->source_index = source_index;
     zip_error_init(&ctx->error);
-    ctx->supports = (zip_source_supports(src) & ZIP_SOURCE_SUPPORTS_SEEKABLE) | (zip_source_make_command_bitmap(ZIP_SOURCE_GET_FILE_ATTRIBUTES, ZIP_SOURCE_SUPPORTS, ZIP_SOURCE_TELL, -1));
+    ctx->supports = (zip_source_supports(src) & (ZIP_SOURCE_SUPPORTS_SEEKABLE | ZIP_SOURCE_SUPPORTS_REOPEN)) | (zip_source_make_command_bitmap(ZIP_SOURCE_GET_FILE_ATTRIBUTES, ZIP_SOURCE_GET_DOS_TIME, ZIP_SOURCE_SUPPORTS, ZIP_SOURCE_TELL, ZIP_SOURCE_FREE, -1));
     ctx->needs_seek = (ctx->supports & ZIP_SOURCE_MAKE_COMMAND_BITMASK(ZIP_SOURCE_SEEK)) ? true : false;
 
     if (st) {
@@ -113,7 +125,11 @@ _zip_source_window_new(zip_source_t *src, zip_uint64_t start, zip_int64_t length
         }
     }
     
-    return zip_source_layered_create(src, window_read, ctx, error);
+    window_source = zip_source_layered_create(src, window_read, ctx, error);
+    if (window_source != NULL && !take_ownership) {
+        zip_source_keep(src);
+    }
+    return window_source;
 }
 
 
@@ -182,7 +198,7 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
             for (n = 0; n < ctx->start; n += (zip_uint64_t)ret) {
                 i = (ctx->start - n > BUFSIZE ? BUFSIZE : ctx->start - n);
                 if ((ret = zip_source_read(src, b, i)) < 0) {
-                    _zip_error_set_from_source(&ctx->error, src);
+                    zip_error_set_from_source(&ctx->error, src);
                     byte_array_fini(b);
                     return -1;
                 }
@@ -210,7 +226,7 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
 
         if (ctx->needs_seek) {
             if (zip_source_seek(src, (zip_int64_t)ctx->offset, SEEK_SET) < 0) {
-                _zip_error_set_from_source(&ctx->error, src);
+                zip_error_set_from_source(&ctx->error, src);
                 return -1;
             }
         }
@@ -241,12 +257,12 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
             }
             if (args->whence == SEEK_END) {
                 if (zip_source_seek(src, args->offset, args->whence) < 0) {
-                    _zip_error_set_from_source(&ctx->error, src);
+                    zip_error_set_from_source(&ctx->error, src);
                     return -1;
                 }
                 new_offset = zip_source_tell(src);
                 if (new_offset < 0) {
-                    _zip_error_set_from_source(&ctx->error, src);
+                    zip_error_set_from_source(&ctx->error, src);
                     return -1;
                 }
                 if ((zip_uint64_t)new_offset < ctx->start) {
@@ -277,6 +293,19 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
         if (_zip_stat_merge(st, &ctx->stat, &ctx->error) < 0) {
             return -1;
         }
+
+        if (!(ctx->stat.valid & ZIP_STAT_SIZE)) {
+            if (ctx->end_valid) {
+                st->valid |= ZIP_STAT_SIZE;
+                st->size = ctx->end - ctx->start;
+            }
+            else if (st->valid & ZIP_STAT_SIZE) {
+                st->size -= ctx->start;
+            }
+        }
+
+        st->valid &= ~ctx->stat_invalid;
+
         return 0;
     }
 
@@ -286,9 +315,22 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
             return -1;
         }
 
-        memcpy(data, &ctx->attributes, sizeof(ctx->attributes));
+        (void)memcpy_s(data, sizeof(ctx->attributes), &ctx->attributes, sizeof(ctx->attributes));
         return sizeof(ctx->attributes);
 
+    case ZIP_SOURCE_GET_DOS_TIME:
+        if (len < sizeof(ctx->dostime)) {
+            zip_error_set(&ctx->error, ZIP_ER_INVAL, 0);
+            return -1;
+        }
+        if (ctx->dostime_valid) {
+            (void)memcpy_s(data, sizeof(ctx->dostime), &ctx->dostime, sizeof(ctx->dostime));
+            return sizeof(ctx->dostime);
+        }
+        else {
+            return 0;
+        }
+
     case ZIP_SOURCE_SUPPORTS:
         return ctx->supports;
 
@@ -296,8 +338,7 @@ window_read(zip_source_t *src, void *_ctx, void *data, zip_uint64_t len, zip_sou
         return (zip_int64_t)(ctx->offset - ctx->start);
 
     default:
-        zip_error_set(&ctx->error, ZIP_ER_OPNOTSUPP, 0);
-        return -1;
+        return zip_source_pass_to_lower_layer(src, data, len, cmd);
     }
 }
 
